// イメージビュア 8x8版
// 公開された8x8ドット イメージデータを一覧・拡大表示します
// ※別途イメージデータファイル(Img8x8.int)が必要です
// STARTボタンを押すと終了します

// イメージデータについて
// http://www.picosoft.co.jp/ORANGE-light/SystemCharacters.pdf
// Copyright (c) 2019 Picosoft Co., Ltd.
// Made by Kaori Hanatsumi

//下記"18"を、IMG8x8.intを保存したEEPROMスロット番号に書き換えてください
FSLOT = 18-1

' 色定数の定義
WHITE = rgb(255, 255, 255)
DARKGRAY = rgb(64, 64, 64)
ORANGE = rgb(255, 163, 0)
LBLUE = rgb(0, 140, 255)
DARKBLUE = rgb(0, 0, 64)

' ボタンの定義
BUTTON_UP = 5
BUTTON_DOWN = 8
BUTTON_LEFT = 7
BUTTON_RIGHT = 6
BUTTON_START = 9

' ワークエリア定義

OFFSET = getmptr()
mptr OFFSET + 2
BUF = getmptr()
mptr BUF + 8
LED_WORK = getmptr()

HT16K33_ADDRESS = 0x70

i2ci 1,400
HT16K33_setup(1)
HT16K33_setBlink(1, 0)
HT16K33_setBrightness(1)
HT16K33_clear()

cls 2
for i=0 to 129
	img_get(i,WHITE,DARKBLUE)
next

gprint 0,232,"Push START to Exit",ORANGE,0

' メインループ
x=0:y=0:img_zoom(0)
while in(BUTTON_START)
	HT16K33_display()
	pre_i = y*10 + x
	x = x + trig_right() - trig_left()
	y = y + trig_down() - trig_up()
	x = x + ((x<0) - (x>9))*10
	y = y + ((y<0) - (y>12))*13
	i = y*10 + x
	if i <> pre_i then img_get(pre_i,WHITE,DARKBLUE):img_zoom(i)
wend
end

' イメージデータを読んで表示
def img_get(n,col1,col2) {
	img_read(n,BUF)
	gput8c(10 + n%10,n/10,BUF,col1,col2)
}

' 読み込み済みのイメージを拡大表示
def img_zoom(i) {
	var cx
	var cy
	var d
	var c
	img_get(i,DARKGRAY,ORANGE)
	cy=7
	while cy>=0
		d=mpeek(BUF+cy)
		cx=7
		while cx>=0
			if d&1 then c=WHITE else c=0
			rect cx*10+1,cy*10+11,8,8,c,1
			d=d>>1:cx=cx-1
		wend
		cy=cy-1
	wend
}

' イメージデータをEEPROMから読み込む
def img_read(id,ptr) {
	mpokew OFFSET/2,FSLOT*2048 + 32 + id*8
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,8,1)
}

' 16ドット単位で8x8イメージデータを表示
def gput8c(x,y,adr,col1,col2) {
	sprnew(0,x*16,y*16,8,8,adr,col1,col2)
	sprdsp 0,1
}

' ボタン状態が変化＆押された時だけ1になる関数
def trig_down() {
	var ret
	ret = pre_d
	pre_d = in(BUTTON_DOWN)
	return ret & !pre_d
}
def trig_up() {
	var ret
	ret = pre_u
	pre_u = in(BUTTON_UP)
	return ret & !pre_u
}
def trig_left() {
	var ret
	ret = pre_l
	pre_l = in(BUTTON_LEFT)
	return ret & !pre_l
}
def trig_right() {
	var ret
	ret = pre_r
	pre_r = in(BUTTON_RIGHT)
	return ret & !pre_r
}

def HT16K33_setup(sw) {
	mpoke LED_WORK, 0x20 | sw
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 1, 1)
	pause 1000
}

def HT16K33_setBlink(sw, f) {
	mpoke LED_WORK, 0x80 | (f << 1) | sw
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 1, 1)
}

def HT16K33_setBrightness(level) {
	mpoke LED_WORK, 0xe0 | level
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 1, 1)
}

def HT16K33_clear() {
	mptr LED_WORK
	mdata 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 17, 1)
}

def HT16K33_display() {
	mptr LED_WORK
	mdata 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	for i = 0 to 7
		s = mpeek(BUF + i)
		d = 0
		if s & 0x80 then d = d | 0x80
		if s & 0x40 then d = d | 0x01
		if s & 0x20 then d = d | 0x02
		if s & 0x10 then d = d | 0x04
		if s & 0x08 then d = d | 0x08
		if s & 0x04 then d = d | 0x10
		if s & 0x02 then d = d | 0x20
		if s & 0x01 then d = d | 0x40
		mpoke LED_WORK + 1 + i * 2, d
	next
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 17, 1)
}
